namespace TCP
{
	public class SocketCreatorClass
	{
		System.Net.Sockets.Socket m_soc;
		public SocketCreatorClass()
		{
			m_soc = new System.Net.Sockets.Socket(	System.Net.Sockets.AddressFamily.InterNetwork,
								System.Net.Sockets.SocketType.Stream,
								System.Net.Sockets.ProtocolType.Tcp);
		}
		public System.Net.Sockets.Socket GetServerSocket(int provided_port)
		{
			System.Net.IPAddress hostip = null;
			foreach(System.Net.IPAddress tempip in (System.Net.Dns.GetHostEntry(System.Net.Dns.GetHostName()).AddressList))
			{
				if(tempip.AddressFamily == System.Net.Sockets.AddressFamily.InterNetwork)
				{
					hostip = tempip;
				}
			}
			System.Net.IPEndPoint endpoint = new System.Net.IPEndPoint(hostip, provided_port);
			m_soc.Bind(endpoint);
			m_soc.Listen(10);
			return m_soc;
		}
		public System.Net.Sockets.Socket GetClientSocket(string provided_servername, int provided_port)
		{
			System.Net.IPAddress ipaddr = null;
			foreach(System.Net.IPAddress tempip in (System.Net.Dns.GetHostEntry(provided_servername).AddressList))
			{
				if(tempip.AddressFamily == System.Net.Sockets.AddressFamily.InterNetwork)
				{
					ipaddr = tempip;
				}
			}
			System.Net.IPEndPoint endpoint = new System.Net.IPEndPoint(ipaddr, provided_port);
			m_soc.Connect(endpoint);
			return m_soc;
		}
	}

	public class AsyncStateObject
	{
		public System.Net.Sockets.Socket Socket;
		public byte[] ReceiveBuffer;
		public System.IO.MemoryStream ReceivedData;
		public AsyncStateObject(System.Net.Sockets.Socket soc)
		{
			this.Socket = soc;
			this.ReceiveBuffer = new byte[1024];
			this.ReceivedData = new System.IO.MemoryStream();
		}
	}

	public class TCPBaseClass
	{
		public TCPBaseClass()
		{
		}
		public void StartReceive(System.Net.Sockets.Socket soc)
		{
			AsyncStateObject so = new AsyncStateObject(soc);
			soc.BeginReceive(	so.ReceiveBuffer,
						0,
						so.ReceiveBuffer.Length,
						System.Net.Sockets.SocketFlags.None,
						new System.AsyncCallback(ReceiveDataCallback),
						so);
		}
		public void ReceiveDataCallback(System.IAsyncResult ar)
		{
			AsyncStateObject so = (AsyncStateObject)ar.AsyncState;
			int len = 0;
			try
			{
				len = so.Socket.EndReceive(ar);
			}
			catch
			{
				return;
			}
			if(0 >= len)
			{
				so.Socket.Close();
				return;
			}
			so.ReceivedData.Write(so.ReceiveBuffer, 0, len);
			TCPMessageInterpreterClass interpreter = null;
			if(0 == so.Socket.Available)
			{
				string str = System.Text.Encoding.UTF8.GetString(so.ReceivedData.ToArray());
				interpreter = new TCPMessageInterpreterClass(str);
				ReceivedAndAct(interpreter, so.Socket);
				so.ReceivedData = new System.IO.MemoryStream();
			}
			if((null != interpreter) && (TCPMessageInterpreterClass.CloseCommand != interpreter.command) || (null == interpreter))
			{
				try
				{
					so.Socket.BeginReceive(	so.ReceiveBuffer,
								0,
								so.ReceiveBuffer.Length,
								System.Net.Sockets.SocketFlags.None,
								new System.AsyncCallback(ReceiveDataCallback),
								so);
				}
				catch
				{
				}
			}
		}
		public virtual void ReceivedAndAct(	TCPMessageInterpreterClass provided_interpreter,
							System.Net.Sockets.Socket provided_socket)
		{
			return;
		}
		public void SendData(	System.Net.Sockets.Socket provided_socket,
					string provided_str)
		{
			provided_socket.Send(System.Text.Encoding.UTF8.GetBytes(provided_str));
			return;
		}
	}
	public class TCPServerClass : TCPBaseClass
	{
		int m_port;
		public int port
		{
			get
			{
				return m_port;
			}
		}
		System.Net.Sockets.Socket m_serversocket;
		public System.Net.Sockets.Socket serversocket
		{
			get
			{
				return m_serversocket;
			}
		}
		ClientListClass m_clientlist;
		public ClientListClass clientlist
		{
			get
			{
				return m_clientlist;
			}
		}
		public TCPServerClass(int provided_port) : base()
		{
			m_port = provided_port;
			m_serversocket = null;
			m_clientlist = new ClientListClass();
		}
		public void StartListen()
		{
			SocketCreatorClass soccreator = new SocketCreatorClass();
			m_serversocket = soccreator.GetServerSocket(m_port);
		}
		public void EndListen()
		{
			m_serversocket.Close();
		}
		public void StartAccept()
		{
			m_serversocket.BeginAccept(new System.AsyncCallback(AcceptCallback), m_serversocket);
		}
		public void AcceptCallback(System.IAsyncResult ar)
		{
			System.Net.Sockets.Socket server = (System.Net.Sockets.Socket)ar.AsyncState;
			System.Net.Sockets.Socket client = null;
			try
			{
				client = server.EndAccept(ar);
			}
			catch
			{
				return;
			}
			StartReceive(client);
			client.Send(System.Text.Encoding.UTF8.GetBytes(TCPMessageInterpreterClass.MakeWhoAreYouCommand()));

			server.BeginAccept(new System.AsyncCallback(AcceptCallback), server);
		}
		public override void ReceivedAndAct(	TCPMessageInterpreterClass provided_interpreter,
							System.Net.Sockets.Socket provided_socket)
		{
			switch(provided_interpreter.command)
			{
				case TCPMessageInterpreterClass.MessageCommand:
					{
						foreach(ClientElementClass tempc in m_clientlist.list)
						{
							tempc.Feed(provided_interpreter.envelope);
						}
						Deliver();
						break;
					}
				case TCPMessageInterpreterClass.CloseCommand:
					{
						int idx = m_clientlist.FindIndex(provided_interpreter.parameter);
						if(-1 != idx)
						{
							provided_socket.Shutdown(System.Net.Sockets.SocketShutdown.Both);
							provided_socket.Close();
						}
						m_clientlist = m_clientlist - provided_interpreter.parameter;
						break;
					}
				case TCPMessageInterpreterClass.MyNameIsCommand:
					{
						ClientElementClass tempclient= new ClientElementClass(	m_clientlist.max_seq + 1,
													provided_interpreter.myname,
													provided_socket);
						m_clientlist = m_clientlist + tempclient;
						Deliver();
						break;
					}
				case TCPMessageInterpreterClass.DiscretionaryCommand:
					{
						break;
					}
				default:
					{
						break;
					}
			}
		}
		public void Deliver()
		{
			foreach(ClientElementClass tempc in m_clientlist.list)
			{
				while(true)
				{
					TCPEnvelopeClass tempenvelope = tempc.envelopelist.Get();
					if(null != tempenvelope)
					{
						SendData(tempc.soc, TCPMessageInterpreterClass.MakeMessageCommand(tempenvelope));
					}
					else
					{
						break;
					}
				}
			}
		}
	}
	public class TCPClientClass : TCPBaseClass
	{
		string m_name;
		public string name
		{
			get
			{
				return m_name;
			}
		}
		string m_server;
		public string server
		{
			get
			{
				return m_server;
			}
		}
		int m_port;
		public int port
		{
			get
			{
				return m_port;
			}
		}
		System.Net.Sockets.Socket m_soc;
		public System.Net.Sockets.Socket soc
		{
			get
			{
				return m_soc;
			}
		}
		TCPEnvelopeListClass m_envelopelist;
		public TCPEnvelopeListClass envelopelist
		{
			get
			{
				return m_envelopelist;
			}
		}
		public TCPClientClass(string provided_server, int provided_port, string provided_name)
		{
			m_server = provided_server;
			m_port = provided_port;
			m_name = provided_name;
			m_envelopelist = new TCPEnvelopeListClass();
		}
		public void Connect()
		{
			SocketCreatorClass tempc = new SocketCreatorClass();
			m_soc = tempc.GetClientSocket(m_server, m_port);
			StartReceive(m_soc);
		}
		public void Send(string provided_to, string provided_message)
		{
			TCPEnvelopeClass tempenvelope = new TCPEnvelopeClass(	m_name,
										provided_to,
										System.DateTime.Now,
										provided_message);
			string tempstr = TCPMessageInterpreterClass.MakeMessageCommand(tempenvelope);
			SendData(tempstr);
		}
		public void SendData(string provided_message)
		{
			m_soc.Send(System.Text.Encoding.UTF8.GetBytes(provided_message));
		}
		public void Close()
		{
			string tempstr = TCPMessageInterpreterClass.MakeCloseCommand(m_name);
			SendData(tempstr);
			try
			{
				m_soc.Shutdown(System.Net.Sockets.SocketShutdown.Both);
			}
			catch
			{
			}
			m_soc.Close();
		}
		public override void ReceivedAndAct(	TCPMessageInterpreterClass provided_interpreter,
							System.Net.Sockets.Socket provided_socket)
		{
			switch(provided_interpreter.command)
			{
				case TCPMessageInterpreterClass.WhoAreYouCommand:
					{
						string tempstr = TCPMessageInterpreterClass.MakeMyNameIsCommand(m_name);
						SendData(tempstr);
						break;
					}
				case TCPMessageInterpreterClass.MessageCommand:
					{
						m_envelopelist = m_envelopelist + provided_interpreter.envelope;
						break;
					}
				case TCPMessageInterpreterClass.DiscretionaryCommand:
					{
						break;
					}
				default:
					{
						break;
					}
			}
		}
	}
	public class ClientElementClass
	{
		int m_seq;
		public int seq
		{
			get
			{
				return m_seq;
			}
		}
		string m_name;
		public string name
		{
			get
			{
				return m_name;
			}
		}
		TCPEnvelopeListClass m_envelopelist;
		public TCPEnvelopeListClass envelopelist
		{
			get
			{
				return m_envelopelist;
			}
		}
		System.Net.Sockets.Socket m_soc;
		public System.Net.Sockets.Socket soc
		{
			get
			{
				return m_soc;
			}
		}
		public ClientElementClass(	int provided_seq,
						string provided_name,
						System.Net.Sockets.Socket provided_socket)
		{
			m_seq = provided_seq;
			m_name = provided_name;
			m_soc = provided_socket;
			m_envelopelist = new TCPEnvelopeListClass();
		}
		public ClientElementClass(	int provided_seq,
						string provided_name,
						System.Net.Sockets.Socket provided_socket,
						TCPEnvelopeListClass provided_envelopelist)
		{
			m_seq = provided_seq;
			m_name = provided_name;
			m_soc = provided_socket;
			m_envelopelist = provided_envelopelist;
		}
		public ClientElementClass(ClientElementClass provided_obj)
		{
			m_seq = provided_obj.seq;
			m_name = provided_obj.name;
			m_soc = provided_obj.soc;
			m_envelopelist = provided_obj.envelopelist;
		}
		public void Feed(TCPEnvelopeClass provided_envelope)
		{
			if(("*" == provided_envelope.to) || (m_name == provided_envelope.to))
			{
				m_envelopelist = m_envelopelist + provided_envelope;
			}
		}
	}
	public class ClientListClass
	{
		ClientElementClass[] m_list;
		public ClientElementClass[] list
		{
			get
			{
				return m_list;
			}
		}
		public int max_seq
		{
			get
			{
				int ret = 0;
				if(null != m_list)
				{
					for(int icnt = 0; icnt < m_list.Length; icnt++)
					{
						if(ret < m_list[icnt].seq)
						{
							ret = m_list[icnt].seq;
						}
					}
				}
				return ret;
			}
		}
		public ClientListClass()
		{
			m_list = null;
		}
		public static ClientListClass operator +(	ClientListClass provided_list,
								ClientElementClass provided_obj)
		{
			ClientListClass ret = provided_list;
			if(-1 == ret.FindIndex(provided_obj))
			{
				ret.JustAdd(provided_obj);
			}
			return ret;
		}
		public static ClientListClass operator -(	ClientListClass provided_list,
								ClientElementClass provided_obj)
		{
			ClientListClass ret = new ClientListClass();
			if(null != provided_list.list)
			{
				for(int icnt = 0; icnt < provided_list.list.Length; icnt++)
				{
					if(provided_obj.name != provided_list.list[icnt].name)
					{
						ret = ret + provided_list.list[icnt];
					}
				}
			}
			return ret;
		}
		public static ClientListClass operator -(	ClientListClass provided_list,
								string provided_name)
		{
			ClientListClass ret = provided_list;
			int idx = provided_list.FindIndex(provided_name);
			if(-1 != idx)
			{
				ret = provided_list - provided_list.list[idx];
			}
			return ret;
		}
		public int FindIndex(ClientElementClass provided_obj)
		{
			return FindIndex(provided_obj.name);
		}
		public int FindIndex(string provided_name)
		{
			if(null == m_list)
			{
				return -1;
			}
			else
			{
				for(int icnt = 0; icnt < m_list.Length; icnt++)
				{
					if(m_list[icnt].name == provided_name)
					{
						return icnt;
					}
				}
				return -1;
			}
		}
		private void JustAdd(ClientElementClass provided_obj)
		{
			if(null == m_list)
			{
				m_list = new ClientElementClass[1];
				m_list[0] = provided_obj;
			}
			else
			{
				ClientElementClass[] templist = new ClientElementClass[m_list.Length + 1];
				for(int icnt = 0; icnt < m_list.Length; icnt++)
				{
					templist[icnt] = m_list[icnt];
				}
				templist[templist.Length -1] = provided_obj;
				m_list = templist;
			}
		}
	}
	public class TCPMessageInterpreterClass
	{
		public const string Delimiter = "*****";
		public const string MessageCommand = "MessageCommand";
		public const string CloseCommand = "CloseCommand";
		public const string WhoAreYouCommand = "WhoAreYouCommand";
		public const string MyNameIsCommand = "MyNameIsCommand";
		public const string DiscretionaryCommand = "DiscretionaryCommand";
		string m_command;
		public string command
		{
			get
			{
				return m_command;
			}
		}
		TCPEnvelopeClass m_envelope;
		public TCPEnvelopeClass envelope
		{
			get
			{
				return m_envelope;
			}
		}
		string m_myname;
		public string myname
		{
			get
			{
				return m_myname;
			}
		}
		string m_parameter;
		public string parameter
		{
			get
			{
				return m_parameter;
			}
		}
		public static string MakeMessageCommand(TCPEnvelopeClass provided_obj)
		{
			string ret = MessageCommand;
			ret += Delimiter;
			ret += provided_obj.Serialize();
			return ret;
		}
		public static string MakeCloseCommand(string provided_myname)
		{
			string ret = CloseCommand;
			ret += Delimiter;
			ret += provided_myname;
			return ret;
		}
		public static string MakeWhoAreYouCommand()
		{
			string ret = WhoAreYouCommand;
			ret += Delimiter;
			ret += Delimiter;
			return ret;
		}
		public static string MakeMyNameIsCommand(string provided_myname)
		{
			string ret = MyNameIsCommand;
			ret += Delimiter;
			ret += provided_myname;
			return ret;
		}
		public static string MakeDiscretionaryCommand(string provided_parameter)
		{
			string ret = DiscretionaryCommand;
			ret += Delimiter;
			ret += provided_parameter;
			return ret;
		}
		public TCPMessageInterpreterClass(string provided_str)
		{
			string work = provided_str;
			int idx = work.IndexOf(Delimiter);
			m_command = work.Substring(0, idx);
			work = work.Substring(idx + Delimiter.Length);
			switch(m_command)
			{
				case MessageCommand:
					{
						m_envelope = new TCPEnvelopeClass(work);
						break;
					}
				case CloseCommand:
					{
						m_parameter = work;
						break;
					}
				case WhoAreYouCommand:
					{
						break;
					}
				case MyNameIsCommand:
					{
						m_myname = work;
						break;
					}
				case DiscretionaryCommand:
					{
						break;
					}
				default:
					{
						break;
					}
			}
		}
	}
	public class TCPEnvelopeListClass
	{
		TCPEnvelopeClass[] m_list;
		public TCPEnvelopeClass[] list
		{
			get
			{
				return m_list;
			}
		}
		public TCPEnvelopeListClass()
		{
			m_list = null;
		}
		public static TCPEnvelopeListClass operator +(	TCPEnvelopeListClass provided_list,
								TCPEnvelopeClass provided_obj)
		{
			TCPEnvelopeListClass ret = provided_list;
			ret.JustAdd(provided_obj);
			return ret;
		}
		public void JustAdd(TCPEnvelopeClass provided_obj)
		{
			if(null == m_list)
			{
				m_list = new TCPEnvelopeClass[1];
				m_list[0] = provided_obj;
			}
			else
			{
				TCPEnvelopeClass[] templist = new TCPEnvelopeClass[m_list.Length + 1];
				for(int icnt = 0; icnt < m_list.Length; icnt++)
				{
					templist[icnt] = m_list[icnt];
				}
				templist[templist.Length - 1] = provided_obj;
				m_list = templist;
			}
		}
		public TCPEnvelopeClass Get()
		{
			TCPEnvelopeClass ret = null;
			if(null != m_list)
			{
				ret = m_list[0];
				if(m_list.Length > 1)
				{
					TCPEnvelopeClass[] templist = new TCPEnvelopeClass[m_list.Length -1];
					for(int icnt = 1; icnt < m_list.Length; icnt++)
					{
						templist[icnt -1] = m_list[icnt];
					}
					m_list = templist;
				}
				else
				{
					m_list = null;
				}
			}
			return ret;
		}
	}
	public class TCPEnvelopeClass
	{
		string m_from;
		public string from
		{
			get
			{
				return m_from;
			}
		}
		string m_to;
		public string to
		{
			get
			{
				return m_to;
			}
		}
		System.DateTime m_date;
		public System.DateTime date
		{
			get
			{
				return m_date;
			}
		}
		string m_message;
		public string message
		{
			get
			{
				return m_message;
			}
		}
		public const string Delimiter = "@@@@@";
		public TCPEnvelopeClass(string provided_str)
		{
			string work = provided_str;
			int idx = work.IndexOf(Delimiter);
			m_from = work.Substring(0, idx);
			work = work.Substring(idx + Delimiter.Length);
			idx = work.IndexOf(Delimiter);
			m_to = work.Substring(0, idx);
			work = work.Substring(idx + Delimiter.Length);
			idx = work.IndexOf(Delimiter);
			string tempdate = work.Substring(0, idx);
			m_message = work.Substring(idx + Delimiter.Length);
			m_date = System.DateTime.Parse(tempdate);
		}
		public TCPEnvelopeClass(	string provided_from,
						string provided_to,
						System.DateTime provided_date,
						string provided_message)
		{
			m_from = provided_from;
			m_to = provided_to;
			m_date = provided_date;
			m_message = provided_message;
		}
		public TCPEnvelopeClass(TCPEnvelopeClass provided_obj)
		{
			m_from = provided_obj.from;
			m_to = provided_obj.to;
			m_date = provided_obj.date;
			m_message = provided_obj.message;
		}
		public string Serialize()
		{
			string ret = "";
			ret += m_from;
			ret += Delimiter;
			ret += m_to;
			ret += Delimiter;
			ret += m_date.ToString();
			ret += Delimiter;
			ret += m_message;
			return ret;
		}
	}
	public class PropertyElementClass
	{
		string m_name;
		public string name
		{
			get
			{
				return m_name;
			}
		}
		string m_value;
		public string value
		{
			get
			{
				return m_value;
			}
		}
		public PropertyElementClass(string provided_name, string provided_value)
		{
			m_name = provided_name;
			m_value = provided_value;
		}
		public PropertyElementClass(PropertyElementClass provided_obj)
		{
			m_name = provided_obj.name;
			m_value = provided_obj.value;
		}
	}
	public class PropertyListClass
	{
		public const string EqualDelimiter = "=====";
		public const string PropertyDelimiter = "<PROPERTYDELIMITER>";
		PropertyElementClass[] m_list;
		public PropertyElementClass[] list
		{
			get
			{
				return m_list;
			}
		}
		public string this[string provided_name]
		{
			get
			{
				string ret = "";
				try
				{
					ret = m_list[FindIndex(provided_name)].value;
				}
				catch
				{
				}
				return ret;
			}
		}
		public PropertyListClass()
		{
			m_list = null;
		}
		public PropertyListClass(PropertyListClass provided_obj)
		{
			m_list = provided_obj.list;
		}
		public PropertyListClass(string provided_str)
		{
			Deserialize(provided_str);
		}
		public static PropertyListClass operator +(	PropertyListClass provided_list,
								PropertyElementClass provided_obj)
		{
			PropertyListClass ret = provided_list;
			if(-1 == provided_list.FindIndex(provided_obj.name))
			{
				ret.JustAdd(provided_obj);
			}
			return ret;
		}
		public static PropertyListClass operator -(	PropertyListClass provided_list,
								PropertyElementClass provided_obj)
		{
			PropertyListClass ret = new PropertyListClass();
			int index = provided_list.FindIndex(provided_obj.name);
			if(null != provided_list.list)
			{
				for(int icnt = 0; icnt < provided_list.list.Length; icnt++)
				{
					if(index != icnt)
					{
						ret = ret + provided_list.list[icnt];
					}
				}
			}
			return ret;
		}
		public string Serialize()
		{
			string ret = "";
			if(null != m_list)
			{
				for(int icnt = 0; icnt < m_list.Length; icnt++)
				{
					ret += (m_list[icnt].name + EqualDelimiter + m_list[icnt].value + PropertyDelimiter);
				}
			}
			return ret;
		}
		public void Deserialize(string provided_str)
		{
			m_list = null;
			string work = provided_str;
			while(true)
			{
				try
				{
					int index = work.IndexOf(PropertyDelimiter);
					string namevalue = work.Substring(0, index);
					int equalindex = namevalue.IndexOf(EqualDelimiter);
					string tempname = namevalue.Substring(0, equalindex);
					string tempvalue = namevalue.Substring(equalindex + EqualDelimiter.Length);
					PropertyElementClass tempprop = new PropertyElementClass(tempname, tempvalue);
					if(-1 == FindIndex(tempname))
					{
						JustAdd(tempprop);
					}
					work = work.Substring(index + PropertyDelimiter.Length);
				}
				catch
				{
					break;
				}
			}
		}
		private int FindIndex(string provided_name)
		{
			if(null != m_list)
			{
				for(int icnt = 0; icnt < m_list.Length; icnt++)
				{
					if(m_list[icnt].name == provided_name)
					{
						return icnt;
					}
				}
			}
			return -1;
		}
		private void JustAdd(PropertyElementClass provided_obj)
		{
			if(null == m_list)
			{
				m_list = new PropertyElementClass[1];
				m_list[0] = provided_obj;
			}
			else
			{
				PropertyElementClass[] templist = new PropertyElementClass[m_list.Length + 1];
				for(int icnt = 0; icnt < m_list.Length; icnt++)
				{
					templist[icnt] = m_list[icnt];
				}
				templist[templist.Length -1] = provided_obj;
				m_list = templist;
			}
		}
	}
}
